/* Copyright (c) Jasper van de Gronde, 2004
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above
 *     copyright notice, this list of conditions and the
 *     following disclaimer.
 *  2. Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;

/** <summary>LUID class</summary>
 * <remarks>Holds a LUID and has a default constructor that creates one.
 * It also implements ToString.
 * LUIDs generated by this class are 2*(sizeof(long)+sizeof(int)) characters long (24 normally).</remarks>
 */

class LUID {
  private string luid;
  
  // A random number is used as starting point to further minimize the chance of two instances generating the same LUID.
  private static int index = (new Random()).Next();

  public LUID() {
    long ticks = DateTime.Now.Ticks;
    luid = convertToHex(BitConverter.GetBytes(ticks))
      + convertToHex(BitConverter.GetBytes(index));
    index++;
  }

  public override string ToString() {
    return luid;
  }

  private static string convertToHex(byte[] bytes) {
    string r = "";
    foreach(byte b in bytes) {
      r += hexDigit((byte)(b & 0x0f));
      r += hexDigit((byte)((b & 0xf0) >> 4));
    }
    return r;
  }

  private static char hexDigit(byte b) {
    if ( b < 10 ) return (char)(b + '0');
    if ( b < 16 ) return (char)(b - 10 + 'a');
    return '_';
  }
}
