/* Copyright (c) Jasper van de Gronde, 2004
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above
 *     copyright notice, this list of conditions and the
 *     following disclaimer.
 *  2. Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System.IO;
using System.Text.RegularExpressions;

/** <summary>Interface to MDDB</summary>
 * <remarks>Provides several methods to work with MDDB.</remarks>
 */

class MDDB {
  // TODO: This should become something like private IConnection db.
  private RdfDbConnection rdfdb;
  private Regex luidregex = new Regex("{LUID[0-9a-zA-Z]+}");

  public MDDB() {
    rdfdb = new RdfDbConnection();
  }

  public MDDB(RdfDbConnection rdfdb_p) {
    rdfdb = rdfdb_p;
  }

  // TODO: This should use some kind of system-wide generic method.
  /// <param name="filename">Full path to a file.</param>
  public bool isInMDDB(string filename) {
    if ( luidregex.IsMatch(Path.GetFileNameWithoutExtension(filename)) ) {
      return true;
    }
    return false;
  }

  // TODO: This should use some kind of system-wide generic method.
  /// <param name="filename">Full path to a file.</param>
  public void addFile(string filename) {
    string dir = Path.GetDirectoryName(filename);
    string name = Path.GetFileNameWithoutExtension(filename);
    string ext = Path.GetExtension(filename);

    System.Console.WriteLine("Adding file: " + name + ext);

    // This isn't correct yet, it doesn't check wether this instance is already in MDDB.
    if ( !isInMDDB(filename) ) {
      // The file isn't in MDDB yet.
      LUID luid = new LUID();
      string newfilename = Path.Combine(dir, name + " {LUID" + luid + "}" + ext);
      File.Move(filename, newfilename);
      System.Console.WriteLine("  New filename: " + Path.GetFileName(newfilename));
    }
    else {
      System.Console.WriteLine("  Skipping (already in MDDB)");
    }
  }

  /// <param name="filenames">An array of full paths to files.</param>
  public void addFiles(string[] filenames) {
    foreach(string filename in filenames) {
      addFile(filename);
    }
  }

  /// <param name="directory">Full path to a directory.</param>
  public void addDirectory(string directory) {
    string[] files = Directory.GetFiles(directory);
    foreach(string file in files) {
      addFile(Path.Combine(directory,file));
    }

    string[] dirs = Directory.GetDirectories(directory);
    foreach(string dir in dirs) {
      addDirectory(Path.Combine(directory,dir));
    }
  }

  /// <param name="directories">An array of full paths to directories.</param>
  public void addDirectories(string[] directories) {
    foreach(string directory in directories) {
      addDirectory(directory);
    }
  }
}
